;----------------------------------------------------------------------------
;				NES Sample1 "HELLO, WORLD!"
;					Copyright (C) 2007, Tekepen
;----------------------------------------------------------------------------
.setcpu		"6502"
.autoimport	on

; iNESヘッダ（このゲームカセットの情報）
.segment "HEADER"
	.byte	$4E, $45, $53, $1A	; "NES" Header
	.byte	$02			; PRG-BANKS
	.byte	$01			; CHR-BANKS
	.byte	$01			; Vetrical Mirror
	.byte	$00
	.byte	$00, $00, $00, $00
	.byte	$00, $00, $00, $00


; 起動したらここから処理実行
.segment "STARTUP"
; リセット割り込み（リセットされたとき，今までの処理を中断してここにジャンプする）
.proc	Reset
	; この三行はおまじないのようにとりあえず書いておく
	sei				; 割り込みという，メイン処理の間に挟まる別処理が起こるのを禁止する
	ldx	#$ff		; スタックポインタの値（スタックは$0200~$02ff番地までの領域で，初期値を$ff（$02ff）に設定）
	txs				; Xレジスタに入れたスタックポインタの値をSレジスタ（スタックレジスタ）にコピー

	; スクリーンオフ（とりあえず$2000と$2001に0を入れれば画面が非表示になる，非表示の間にPPUの操作を行う）
	lda	#$00		; lda = LoaD A，Aレジスタに右側の値を代入する
	sta	$2000		; sta = STore A，Ａレジスタの値を右側のアドレスに代入する
	sta	$2001

	; パレットテーブルへ転送(BG用のみ転送)
	lda	#$3f		; #が付いた値は即値という
	sta	$2006		; PPUのRAMアドレス（上位1バイト）
	lda	#$00		; $は16進数，何もなしは10進数，%は2進数を表す
	sta	$2006		; PPUのRAMアドレス（下位1バイト）
	ldx	#$00
	ldy	#$10
; このcopypalをラベルという
; このラベルを使ってジャンプ（プログラムの実行位置を飛ばす）や，ループ処理を行える
copypal:
	lda	palettes, x	; 高級言語的に書くと A = palettes[x]
	sta	$2007		; $3f00から順（$3f00のあとは$3f01，そのあとは$3f02...）にストアされていく（ストア＝RAMへの書き込み）
	inx				; Ｘのインクリメント
	dey				; Yのデクリメント
	bne	copypal		; Yが0でないとき（$10から$1までの間）はcopypalに戻る（ループする）

	; ネームテーブル（背景のデータを格納するテーブル（メモリー領域）へ転送($21c9だと画面の中央付近になる)）
	lda	#$21
	sta	$2006
	lda	#$c9
	sta	$2006
	ldx	#$00
	ldy	#$0d		; 13文字表示のためのループカウンタ（ldy #13と10進数で書いてもよい）
copymap:
	lda	string, x
	sta	$2007		; $21c9にHELLOのH，$21caにE，$21cbにLがストアされていく
	inx
	dey
	bne	copymap

	; スクロール設定（カメラを中央に持ってくる）
	lda	#$00
	sta	$2005		; カメラX座標
	sta	$2005		; カメラY座標

	; スクリーンオン
	lda	#$08		; 3ビット目が1になっている＝スプライト用CHRテーブルの設定（NES研究所参照）
	sta	$2000
	lda	#$1e		; 背景・スプライトを表示，画面の左端を表示
	sta	$2001

	; 無限ループ（処理終わり）
	mainloop:
		jmp	mainloop
.endproc

; パレットテーブル
palettes:
	; 背景色，色１，色２，色３の順番
	; このセットを背景は4つ使い分けられる
	.byte	$0f, $00, $10, $20		; セット1
	.byte	$0f, $06, $16, $26		; セット2
	.byte	$0f, $08, $18, $28		; セット3
	.byte	$0f, $0a, $1a, $2a		; セット4

; 表示文字列
string:
	.byte	"HELLO, WORLD!"

; ROMの最後の領域に割り込み処理のためのアドレスを指定（リセットボタンがおされたらResetまでジャンプする）
.segment "VECINFO"
	.word	$0000
	.word	Reset
	.word	$0000

; キャラクタテーブル（スプライト，背景の画像が入っている）
.segment "CHARS"
	.incbin	"character.chr"
